'----------------------------------------------------------------------------------------------
'Simulator of invertible and non-invertible MA(q) models
'----------------------------------------------------------------------------------------------
'%parameters="-7 -1 2"
%parameters=@equaloption("coefs")
%roots=@equaloption("roots")
scalar correl=0	
scalar equations=0
%n="100000"
if %parameters+%roots="" then
	if @uidialog("Caption","Non-invertible MA(q) simulator","edit",%parameters,"Coefficients of the MA(Q):","text","Estimate: ","Check",correl,"Correlograms","Check",equations,"MA(Q) equations","edit",%roots,"Inverse Real Roots of the MA(Q) polynomial (instead of coefficients):","edit",%n,"Observations to estimate roots (if not provided)","text","Written by: Nicolas Ronderos (2022).")=-1 then
		d correl equations
		stop
	endif
	else
endif
correl=@hasoption("correl")
equations=@hasoption("equations")
if @equaloption("n")<>"" then
	%n=@equaloption("n")
endif
pagestruct(end={%n})
if %roots="" then
	!Q=@wcount(%parameters)
	else
	!Q=@wcount(%roots)
endif
'----------------------------------------------
'Inverse-Roots if not provided
'----------------------------------------------
rndseed 4567
genr e=nrnd
if %roots="" then
	%ma=""
	for !j=1 to !Q
		if !j=!Q then
			%ma=%ma+@word(%parameters,!j)+"*e(-"+@str(!j)+")+e"
		else
			%ma=%ma+@word(%parameters,!j)+"*e(-"+@str(!j)+")+"
		endif
	next
	genr x={%ma}
	equation eq01.ls x c ma(1to!Q)
	freeze(roots) eq01.arma(table)
	vector(!Q) inverse_roots=0
	for !i=1 to !Q
		if @wfind(roots(8+!i,1),"")=2 then
			@uiprompt("Complex roots, try a different set of coefficients")
			d x e eq01 inverse_roots roots
			stop
		endif
	next
	for !j=1 to !Q
		inverse_roots(!j)=roots(8+!j,1)
	next
	matrix(!Q,2) roots_and_inverse=NA
	vector(!Q) root=@einv(inverse_roots)
	colplace(roots_and_inverse,inverse_roots,1)
	colplace(roots_and_inverse,root,2)
	d roots x eq01
	else
	matrix(!Q,2) roots_and_inverse=NA
	vector(!Q) inverse_roots=0
	for !i=1 to !Q
		inverse_roots(!i)=@word(%roots,!i)
	next
	vector(!Q) root=@einv(inverse_roots)
	colplace(roots_and_inverse,inverse_roots,1)
	colplace(roots_and_inverse,root,2)
endif
'----------------------------------------------------------------------------------------------
'Parameters of any (non-invertible or invertible) representation
'----------------------------------------------------------------------------------------------
'Possible sets of roots of any (non-invertible or invertible) MA(Q) model
if !Q=1 then '2 possible sets of roots
	!rep=0
	!loop=1
	for !i=1 to 2
		!rep=!rep+1
		vector(!Q) rep!rep(!loop)=roots_and_inverse(1,!i)
	next
endif
if !Q=2 then '4 possible sets of roots
	!rep=0
	for !i=1 to 2
		!loop=1
		for !j=1 to 2
			!rep=!rep+1
			vector(!Q) rep!rep(!loop)=roots_and_inverse(1,!i)
			!loop=!loop+1
			vector(!Q) rep!rep(!loop)=roots_and_inverse(2,!j)	
			!loop=1	
		next
	next
endif
if !Q=3 then '8 possible sets of roots
	!rep=0
	for !i=1 to 2
		!loop=1
		for !j=1 to 2
			for !h=1 to 2
				!rep=!rep+1
				vector(!Q) rep!rep(!loop)=roots_and_inverse(1,!i)
				!loop=!loop+1
				vector(!Q) rep!rep(!loop)=roots_and_inverse(2,!j)	
				!loop=!loop+1
				vector(!Q) rep!rep(!loop)=roots_and_inverse(3,!h)	
				!loop=1	
			next
		next
	next
endif
if !Q=4 then '16 possible sets of roots
	!rep=0
	for !i=1 to 2
		!loop=1
		for !j=1 to 2
			for !h=1 to 2
				for !u=1 to 2
					!rep=!rep+1
					vector(!Q) rep!rep(!loop)=roots_and_inverse(1,!i)
					!loop=!loop+1
					vector(!Q) rep!rep(!loop)=roots_and_inverse(2,!j)	
					!loop=!loop+1
					vector(!Q) rep!rep(!loop)=roots_and_inverse(3,!h)	
					!loop=!loop+1
					vector(!Q) rep!rep(!loop)=roots_and_inverse(4,!u)
					!loop=1	
				next
			next
		next
	next
endif
if !Q=5 then '32 possible sets of roots
	!rep=0
	for !i=1 to 2
		!loop=1
		for !j=1 to 2
			for !h=1 to 2
				for !u=1 to 2
					for !y=1 to 2
						!rep=!rep+1
						vector(!Q) rep!rep(!loop)=roots_and_inverse(1,!i)
						!loop=!loop+1
						vector(!Q) rep!rep(!loop)=roots_and_inverse(2,!j)	
						!loop=!loop+1
						vector(!Q) rep!rep(!loop)=roots_and_inverse(3,!h)	
						!loop=!loop+1
						vector(!Q) rep!rep(!loop)=roots_and_inverse(4,!u)
						!loop=!loop+1
						vector(!Q) rep!rep(!loop)=roots_and_inverse(5,!y)
						!loop=1	
					next
				next
			next
		next
	next
endif
if !Q=6 then '64 possible sets of roots
	!rep=0
	for !i=1 to 2
		!loop=1
		for !j=1 to 2
			for !h=1 to 2
				for !u=1 to 2
					for !y=1 to 2
						for !o=1 to 2
							!rep=!rep+1
							vector(!Q) rep!rep(!loop)=roots_and_inverse(1,!i)
							!loop=!loop+1
							vector(!Q) rep!rep(!loop)=roots_and_inverse(2,!j)	
							!loop=!loop+1
							vector(!Q) rep!rep(!loop)=roots_and_inverse(3,!h)	
							!loop=!loop+1
							vector(!Q) rep!rep(!loop)=roots_and_inverse(4,!u)
							!loop=!loop+1
							vector(!Q) rep!rep(!loop)=roots_and_inverse(5,!y)
							!loop=!loop+1
							vector(!Q) rep!rep(!loop)=roots_and_inverse(6,!o)
							!loop=1
						next	
					next
				next
			next
		next
	next
endif
if !Q=7 then '128 possible sets of roots
	!rep=0
	for !i=1 to 2
		!loop=1
		for !j=1 to 2
			for !h=1 to 2
				for !u=1 to 2
					for !y=1 to 2
						for !o=1 to 2
							for !p=1 to 2
								!rep=!rep+1
								vector(!Q) rep!rep(!loop)=roots_and_inverse(1,!i)
								!loop=!loop+1
								vector(!Q) rep!rep(!loop)=roots_and_inverse(2,!j)	
								!loop=!loop+1
								vector(!Q) rep!rep(!loop)=roots_and_inverse(3,!h)	
								!loop=!loop+1
								vector(!Q) rep!rep(!loop)=roots_and_inverse(4,!u)
								!loop=!loop+1
								vector(!Q) rep!rep(!loop)=roots_and_inverse(5,!y)
								!loop=!loop+1
								vector(!Q) rep!rep(!loop)=roots_and_inverse(6,!o)
								!loop=!loop+1
								vector(!Q) rep!rep(!loop)=roots_and_inverse(7,!p)
								!loop=1
							next
						next	
					next
				next
			next
		next
	next
endif
if !Q=8 then '256 possible sets of roots
	!rep=0
	for !i=1 to 2
		!loop=1
		for !j=1 to 2
			for !h=1 to 2
				for !u=1 to 2
					for !y=1 to 2
						for !o=1 to 2
							for !p=1 to 2
								for !k=1 to 2
									!rep=!rep+1
									vector(!Q) rep!rep(!loop)=roots_and_inverse(1,!i)
									!loop=!loop+1
									vector(!Q) rep!rep(!loop)=roots_and_inverse(2,!j)	
									!loop=!loop+1
									vector(!Q) rep!rep(!loop)=roots_and_inverse(3,!h)	
									!loop=!loop+1
									vector(!Q) rep!rep(!loop)=roots_and_inverse(4,!u)
									!loop=!loop+1
									vector(!Q) rep!rep(!loop)=roots_and_inverse(5,!y)
									!loop=!loop+1
									vector(!Q) rep!rep(!loop)=roots_and_inverse(6,!o)
									!loop=!loop+1
									vector(!Q) rep!rep(!loop)=roots_and_inverse(7,!p)
									!loop=!loop+1
									vector(!Q) rep!rep(!loop)=roots_and_inverse(8,!k)
									!loop=1
								next
							next
						next	
					next
				next
			next
		next
	next
endif
if !Q=9 then '512 possible sets of roots
	!rep=0
	for !i=1 to 2
		!loop=1
		for !j=1 to 2
			for !h=1 to 2
				for !u=1 to 2
					for !y=1 to 2
						for !o=1 to 2
							for !p=1 to 2
								for !k=1 to 2
									for !l=1 to 2
										!rep=!rep+1
										vector(!Q) rep!rep(!loop)=roots_and_inverse(1,!i)
										!loop=!loop+1
										vector(!Q) rep!rep(!loop)=roots_and_inverse(2,!j)	
										!loop=!loop+1
										vector(!Q) rep!rep(!loop)=roots_and_inverse(3,!h)	
										!loop=!loop+1
										vector(!Q) rep!rep(!loop)=roots_and_inverse(4,!u)
										!loop=!loop+1
										vector(!Q) rep!rep(!loop)=roots_and_inverse(5,!y)
										!loop=!loop+1
										vector(!Q) rep!rep(!loop)=roots_and_inverse(6,!o)
										!loop=!loop+1
										vector(!Q) rep!rep(!loop)=roots_and_inverse(7,!p)
										!loop=!loop+1
										vector(!Q) rep!rep(!loop)=roots_and_inverse(8,!k)
										!loop=!loop+1
										vector(!Q) rep!rep(!loop)=roots_and_inverse(9,!k)
										!loop=1
									next
								next
							next
						next	
					next
				next
			next
		next
	next
endif
if !Q=10 then '1024 possible sets of roots
	!rep=0
	for !i=1 to 2
		!loop=1
		for !j=1 to 2
			for !h=1 to 2
				for !u=1 to 2
					for !y=1 to 2
						for !o=1 to 2
							for !p=1 to 2
								for !k=1 to 2
									for !l=1 to 2
										for !m=1 to 2
											!rep=!rep+1
											vector(!Q) rep!rep(!loop)=roots_and_inverse(1,!i)
											!loop=!loop+1
											vector(!Q) rep!rep(!loop)=roots_and_inverse(2,!j)	
											!loop=!loop+1
											vector(!Q) rep!rep(!loop)=roots_and_inverse(3,!h)	
											!loop=!loop+1
											vector(!Q) rep!rep(!loop)=roots_and_inverse(4,!u)
											!loop=!loop+1
											vector(!Q) rep!rep(!loop)=roots_and_inverse(5,!y)
											!loop=!loop+1
											vector(!Q) rep!rep(!loop)=roots_and_inverse(6,!o)
											!loop=!loop+1
											vector(!Q) rep!rep(!loop)=roots_and_inverse(7,!p)
											!loop=!loop+1
											vector(!Q) rep!rep(!loop)=roots_and_inverse(8,!k)
											!loop=!loop+1
											vector(!Q) rep!rep(!loop)=roots_and_inverse(9,!k)
											!loop=!loop+1
											vector(!Q) rep!rep(!loop)=roots_and_inverse(10,!m)
											!loop=1
										next
									next
								next
							next
						next	
					next
				next
			next
		next
	next
endif
if !Q=11 then '2048 possible sets of roots
	!rep=0
	for !i=1 to 2
		!loop=1
		for !j=1 to 2
			for !h=1 to 2
				for !u=1 to 2
					for !y=1 to 2
						for !o=1 to 2
							for !p=1 to 2
								for !k=1 to 2
									for !l=1 to 2
										for !m=1 to 2
											for !n=1 to 2
												!rep=!rep+1
												vector(!Q) rep!rep(!loop)=roots_and_inverse(1,!i)
												!loop=!loop+1
												vector(!Q) rep!rep(!loop)=roots_and_inverse(2,!j)	
												!loop=!loop+1
												vector(!Q) rep!rep(!loop)=roots_and_inverse(3,!h)	
												!loop=!loop+1
												vector(!Q) rep!rep(!loop)=roots_and_inverse(4,!u)
												!loop=!loop+1
												vector(!Q) rep!rep(!loop)=roots_and_inverse(5,!y)
												!loop=!loop+1
												vector(!Q) rep!rep(!loop)=roots_and_inverse(6,!o)
												!loop=!loop+1
												vector(!Q) rep!rep(!loop)=roots_and_inverse(7,!p)
												!loop=!loop+1
												vector(!Q) rep!rep(!loop)=roots_and_inverse(8,!k)
												!loop=!loop+1
												vector(!Q) rep!rep(!loop)=roots_and_inverse(9,!k)
												!loop=!loop+1
												vector(!Q) rep!rep(!loop)=roots_and_inverse(10,!m)
												!loop=!loop+1
												vector(!Q) rep!rep(!loop)=roots_and_inverse(11,!n)
												!loop=1
											next
										next
									next
								next
							next
						next	
					next
				next
			next
		next
	next
endif
if !Q=12 then '4096 possible sets of roots
	!rep=0
	for !i=1 to 2
		!loop=1
		for !j=1 to 2
			for !h=1 to 2
				for !u=1 to 2
					for !y=1 to 2
						for !o=1 to 2
							for !p=1 to 2
								for !k=1 to 2
									for !l=1 to 2
										for !m=1 to 2
											for !n=1 to 2
												for !b=1 to 2
													!rep=!rep+1
													vector(!Q) rep!rep(!loop)=roots_and_inverse(1,!i)
													!loop=!loop+1
													vector(!Q) rep!rep(!loop)=roots_and_inverse(2,!j)	
													!loop=!loop+1
													vector(!Q) rep!rep(!loop)=roots_and_inverse(3,!h)	
													!loop=!loop+1
													vector(!Q) rep!rep(!loop)=roots_and_inverse(4,!u)
													!loop=!loop+1
													vector(!Q) rep!rep(!loop)=roots_and_inverse(5,!y)
													!loop=!loop+1
													vector(!Q) rep!rep(!loop)=roots_and_inverse(6,!o)
													!loop=!loop+1
													vector(!Q) rep!rep(!loop)=roots_and_inverse(7,!p)
													!loop=!loop+1
													vector(!Q) rep!rep(!loop)=roots_and_inverse(8,!k)
													!loop=!loop+1
													vector(!Q) rep!rep(!loop)=roots_and_inverse(9,!k)
													!loop=!loop+1
													vector(!Q) rep!rep(!loop)=roots_and_inverse(10,!m)
													!loop=!loop+1
													vector(!Q) rep!rep(!loop)=roots_and_inverse(11,!n)
													!loop=1
												next
											next
										next
									next
								next
							next
						next	
					next
				next
			next
		next
	next
endif
'Parameters of any of the 2^Q possible representations of the MA (one is invertible)
scalar representations=2^!Q
for !r=1 to representations
	'B1
	if !Q>=1 then
		scalar B1_!r=-@sum(rep!r)
	endif
	'B2
	if !Q>=2 then
	scalar B2_!r=0
		for !i=1 to !Q
			for !j=1 to !Q
				if !i>!j then
					scalar B2_!r=B2_!r+rep!r(!i)*rep!r(!j)
				endif
			next
		next
	endif
	if !Q>=3 then
	scalar B3_!r=0
		for !i=1 to !Q
			for !j=1 to !Q
				for !h=1 to !Q
					if !i>!j then
						if !j>!h then
							scalar B3_!r=B3_!r-rep!r(!i)*rep!r(!j)*rep!r(!h)
						endif
					endif
				next
			next
		next
	endif
	if !Q>=4 then
	scalar B4_!r=0
		for !i=1 to !Q
			for !j=1 to !Q
				for !h=1 to !Q
					for !u=1 to !Q
						if !i>!j then
							if !j>!h then
								if !h>!u then
									scalar B4_!r=B4_!r-rep!r(!i)*rep!r(!j)*rep!r(!h)*rep!r(!u)
								endif
							endif
						endif
					next
				next
			next
		next
	endif
	if !Q>=5 then
	scalar B5_!r=0
		for !i=1 to !Q
			for !j=1 to !Q
				for !h=1 to !Q
					for !u=1 to !Q
						for !y=1 to !Q
							if !i>!j then
								if !j>!h then
									if !h>!u then
										if !u>!y then
											scalar B5_!r=B5_!r-rep!r(!i)*rep!r(!j)*rep!r(!h)*rep!r(!u)*rep!r(!y)
										endif
									endif
								endif
							endif
						next
					next
				next
			next
		next
	endif
	if !Q>=6 then
	scalar B6_!r=0
		for !i=1 to !Q
			for !j=1 to !Q
				for !h=1 to !Q
					for !u=1 to !Q
						for !y=1 to !Q
							for !k=1 to !Q
								if !i>!j then
									if !j>!h then
										if !h>!u then
											if !u>!y then
												if !y>!k then
													scalar B6_!r=B6_!r-rep!r(!i)*rep!r(!j)*rep!r(!h)*rep!r(!u)*rep!r(!y)*rep!r(!k)
												endif
											endif
										endif
									endif
								endif
							next
						next
					next
				next
			next
		next
	endif
	if !Q>=7 then
	scalar B7_!r=0
		for !i=1 to !Q
			for !j=1 to !Q
				for !h=1 to !Q
					for !u=1 to !Q
						for !y=1 to !Q
							for !k=1 to !Q
								for !l=1 to !Q
									if !i>!j then
										if !j>!h then
											if !h>!u then
												if !u>!y then
													if !y>!k then
														if !k>!l then
															scalar B7_!r=B7_!r-rep!r(!i)*rep!r(!j)*rep!r(!h)*rep!r(!u)*rep!r(!y)*rep!r(!k)*rep!r(!l)
														endif
													endif
												endif
											endif
										endif
									endif
								next
							next
						next
					next
				next
			next
		next
	endif
	if !Q>=8 then
	scalar B8_!r=0
		for !i=1 to !Q
			for !j=1 to !Q
				for !h=1 to !Q
					for !u=1 to !Q
						for !y=1 to !Q
							for !k=1 to !Q
								for !l=1 to !Q
									for !o=1 to !Q
										if !i>!j then
											if !j>!h then
												if !h>!u then
													if !u>!y then
														if !y>!k then
															if !k>!l then
																if !l>!o then
																	scalar B8_!r=B8_!r-rep!r(!i)*rep!r(!j)*rep!r(!h)*rep!r(!u)*rep!r(!y)*rep!r(!k)*rep!r(!l)*rep!r(!o)
																endif
															endif
														endif
													endif
												endif
											endif
										endif
									next
								next
							next
						next
					next
				next
			next
		next
	endif
	if !Q>=9 then
	scalar B9_!r=0
		for !i=1 to !Q
			for !j=1 to !Q
				for !h=1 to !Q
					for !u=1 to !Q
						for !y=1 to !Q
							for !k=1 to !Q
								for !l=1 to !Q
									for !o=1 to !Q
										for !p=1 to !Q
											if !i>!j then
												if !j>!h then
													if !h>!u then
														if !u>!y then
															if !y>!k then
																if !k>!l then
																	if !l>!o then
																		if !o>!p then
																			scalar B9_!r=B9_!r-rep!r(!i)*rep!r(!j)*rep!r(!h)*rep!r(!u)*rep!r(!y)*rep!r(!k)*rep!r(!l)*rep!r(!o)*rep!r(!p)
																		endif
																	endif
																endif
															endif
														endif
													endif
												endif
											endif
										next
									next
								next
							next
						next
					next
				next
			next
		next
	endif
	if !Q>=10 then
	scalar B10_!r=0
		for !i=1 to !Q
			for !j=1 to !Q
				for !h=1 to !Q
					for !u=1 to !Q
						for !y=1 to !Q
							for !k=1 to !Q
								for !l=1 to !Q
									for !o=1 to !Q
										for !p=1 to !Q
											for !t=1 to !Q
												if !i>!j then
													if !j>!h then
														if !h>!u then
															if !u>!y then
																if !y>!k then
																	if !k>!l then
																		if !l>!o then
																			if !o>!p then
																				if !p>!t then
																					scalar B10_!r=B10_!r-rep!r(!i)*rep!r(!j)*rep!r(!h)*rep!r(!u)*rep!r(!y)*rep!r(!k)*rep!r(!l)*rep!r(!o)*rep!r(!p)*rep!r(!t)
																				endif
																			endif
																		endif
																	endif
																endif
															endif
														endif
													endif
												endif
											next
										next
									next
								next
							next
						next
					next
				next
			next
		next
	endif
	if !Q>=11 then
	scalar B11_!r=0
		for !i=1 to !Q
			for !j=1 to !Q
				for !h=1 to !Q
					for !u=1 to !Q
						for !y=1 to !Q
							for !k=1 to !Q
								for !l=1 to !Q
									for !o=1 to !Q
										for !p=1 to !Q
											for !t=1 to !Q
												for !g=1 to !Q
													if !i>!j then
														if !j>!h then
															if !h>!u then
																if !u>!y then
																	if !y>!k then
																		if !k>!l then
																			if !l>!o then
																				if !o>!p then
																					if !p>!t then
																						if !t>!g then
																							scalar B11_!r=B11_!r-rep!r(!i)*rep!r(!j)*rep!r(!h)*rep!r(!u)*rep!r(!y)*rep!r(!k)*rep!r(!l)*rep!r(!o)*rep!r(!p)*rep!r(!t)*rep!r(!g)
																						endif
																					endif
																				endif
																			endif
																		endif
																	endif
																endif
															endif
														endif
													endif
												next
											next
										next
									next
								next
							next
						next
					next
				next
			next
		next
	endif
	if !Q>=12 then
	scalar B12_!r=0
		for !i=1 to !Q
			for !j=1 to !Q
				for !h=1 to !Q
					for !u=1 to !Q
						for !y=1 to !Q
							for !k=1 to !Q
								for !l=1 to !Q
									for !o=1 to !Q
										for !p=1 to !Q
											for !t=1 to !Q
												for !g=1 to !Q
													for !r=1 to !Q
														if !i>!j then
															if !j>!h then
																if !h>!u then
																	if !u>!y then
																		if !y>!k then
																			if !k>!l then
																				if !l>!o then
																					if !o>!p then
																						if !p>!t then
																							if !t>!g then
																								if !g>!r then
																									scalar B12_!r=B12_!r-rep!r(!i)*rep!r(!j)*rep!r(!h)*rep!r(!u)*rep!r(!y)*rep!r(!k)*rep!r(!l)*rep!r(!o)*rep!r(!p)*rep!r(!t)*rep!r(!g)*rep!r(!r)
																								endif
																							endif
																						endif
																					endif
																				endif
																			endif
																		endif
																	endif
																endif
															endif
														endif
													next
												next
											next
										next
									next
								next
							next
						next
					next
				next
			next
		next
	endif
next
'---------------------------------------------------------------------------------------------------
'Simulation of the equivalent (second order moments) MA(q) time series
'----------------------------------------------------------------------------------------------------
%simulation="e+"
for !s=1 to representations
	for !i=1 to !Q
		if !i=!Q then
			%simulation=%simulation+"b"+@str(!i)+"_"+@str(!s)+"*e(-"+@str(!i)+")"
		else
			%simulation=%simulation+"b"+@str(!i)+"_"+@str(!s)+"*e(-"+@str(!i)+")+"
		endif
	next
	genr x!s={%simulation}
	%simulation="e+"
	if correl=1 then
		freeze(c_x!s) x!s.correl
	endif
	if equations=1 then
		equation eq_x!s.ls x!s c ma(1to!Q)
	endif
next
'---------------------------------------------------------
'Representation and roots table
'---------------------------------------------------------
%table = "representations"
if @isobject(%table) then 
	%table = @getnextname(%table)
endif
table(3+representations,3) {%table}
setline({%table},1)
setline({%table},3)
setline({%table},4+representations)
{%table}(2,1)="Representation"
{%table}(2,2)="MA(Q) equation"
{%table}(2,3)="Inverse roots"
setcolwidth({%table},1,14)
setcolwidth({%table},2,8+8*!Q)
setcolwidth({%table},3,8+4*!Q)
for !i=1 to representations
	{%table}(3+!i,1)=!i
	%equation="X"+@str(!i)+"t=et+"
	%roots="("
	for !j=1 to !Q
		if !j=!Q then
			%equation=%equation+@str(@round(b!j_!i,2))+"e(-"+@str(!j)+")"
			%roots=%roots+@str(@round(rep!i(!j),2))+")"
		else
			%roots=%roots+@str(@round(rep!i(!j),2))+","
			%equation=%equation+@str(@round(b!j_!i,2))+"e(-"+@str(!j)+")+"
		endif
	next
	{%table}(3+!i,2)=%equation
	{%table}(3+!i,3)=%roots
next
'Delete
for !j=1 to representations
	for !i=1 to !Q
		d b!i_!j
	next
	d rep!j
next
d inverse_roots roots_and_inverse root representations correl equations


